#!/usr/bin/env python
import logging
import argparse


class Command(object):

    def __init__(self):
        parser = argparse.ArgumentParser(
            description="Taming the bull: Change management for Agresso systems")
        self._logger = logging.getLogger(__name__)
        self._add_arguments(parser)
        self.args, unknown = parser.parse_known_args()
        self._execute()

    def _add_arguments(self, parser):
        pass

    def _execute(self):
        raise NotImplementedError



#!/usr/bin/env python
import os
import subprocess
from matador.session import Session
from .deployment import DeploymentCommand, substitute_keywords
from matador.commands.run_sql_script import run_sql_script


def _checkout_script(path, commit):
    repo_folder = Session.matador_repository_folder
    scriptPath = os.path.join(repo_folder, path)
    targetScript = os.path.join(
        Session.ticket_folder, os.path.basename(scriptPath))

    subprocess.run(
        ['git', '-C', repo_folder, 'checkout', commit],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'),
        check=True)

    originalFile = open(scriptPath, 'r')
    originalText = originalFile.read()
    newText = substitute_keywords(
        originalText, repo_folder, commit)
    originalFile.close()

    newFile = open(targetScript, 'w')
    newFile.write(newText)
    newFile.close()

    return targetScript


class DeploySqlScript(DeploymentCommand):

    def _execute(self):
        path = self.args[0]

        if len(os.path.dirname(path)) == 0:
            targetScript = os.path.join(Session.ticket_folder, path)
        else:
            commit = self.args[1]
            targetScript = _checkout_script(path, commit)

        run_sql_script(self._logger, targetScript)


class DeployOraclePackage(DeploymentCommand):

    def _execute(self):
        packageName = self.args[0]
        commit = self.args[1]

        repo_folder = Session.matador_repository_folder
        package = os.path.join(
            repo_folder, 'src', 'packages', packageName, packageName)
        packageSpec = package + '.pks'
        packageBody = package + '.pkb'

        packageSpecScript = _checkout_script(packageSpec, commit)
        packageBodyScript = _checkout_script(packageBody, commit)

        run_sql_script(self._logger, packageSpecScript)
        run_sql_script(self._logger, packageBodyScript)



#!/usr/bin/env python
from .command import Command
from matador.commands.deployment import *
from matador.session import Session
import subprocess
import os
import shutil


class ActionTicket(Command):
    action = 'None'

    def _add_arguments(self, parser):
        parser.prog = 'matador deploy-ticket'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

        parser.add_argument(
            '-c', '--commit',
            type=str,
            default='none',
            help='Branch name')

        parser.add_argument(
            '-p', '--packaged',
            type=bool,
            default=False,
            help='Whether this deployment is part of a package')

    def _checkout_ticket(self, repo_folder, ticket_folder, commit):

        subprocess.run([
            'git', '-C', repo_folder, 'checkout', commit],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'),
            check=True)

        src = os.path.join(repo_folder, 'deploy', 'tickets', self.args.ticket)
        shutil.copytree(src, ticket_folder)

    def _cleanup(self, ticket_folder):
        shutil.rmtree(ticket_folder)

    def _execute(self):
        Session.set_environment(self.args.environment)
        proj_folder = Session.project_folder
        repo_folder = Session.matador_repository_folder
        ticket_folder = os.path.join(
            Session.matador_tickets_folder, self.args.ticket)
        Session.ticket_folder = ticket_folder

        if not self.args.packaged:
            Session.update_repository()

        if self.args.commit == 'none':
            commit = subprocess.check_output(
                ['git', '-C', proj_folder, 'rev-parse', 'HEAD'],
                stderr=subprocess.STDOUT).decode('utf-8').strip('\n')
        else:
            commit = self.args['commit']

        self._logger.debug(commit)

        self._checkout_ticket(repo_folder, ticket_folder, commit)

        actionFile = self.action + '.py'
        sourceFile = os.path.join(ticket_folder, actionFile)
        try:
            from importlib.machinery import SourceFileLoader
            SourceFileLoader(self.action, sourceFile).load_module()
        finally:
            self._cleanup(ticket_folder)


class DeployTicket(ActionTicket):
    action = 'deploy'


class RemoveTicket(ActionTicket):
    action = 'remove'



#!/usr/bin/env python
import os
import subprocess
import yaml


def get_environments(project_folder):
        file_path = os.path.join(
            project_folder, 'config', 'environments.yml')
        file = open(file_path, 'r')
        return yaml.load(file)


def get_credentials(project_folder):
    file_path = os.path.join(
        project_folder, 'config', 'credentials.yml')
    file = open(file_path, 'r')
    return yaml.load(file)


def initialise_repository(proj_folder, repo_folder):
    subprocess.run([
        'git', '-C', repo_folder, 'init'],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))

    subprocess.run([
        'git', '-C', repo_folder, 'config', 'core.sparsecheckout', 'true'],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))

    subprocess.run([
        'git', '-C', repo_folder, 'remote', 'add', 'origin', proj_folder],
        stderr=subprocess.STDOUT,
        stdout=open(os.devnull, 'w'))

    git_path = (os.path.join(repo_folder, '.git'))

    sparse_checkout_file = os.path.join(
        git_path, 'info', 'sparse-checkout')
    with open(sparse_checkout_file, 'a') as f:
        f.write('/src\n')
        f.write('/deploy\n')
        f.close()


class Session(object):

    environment = None

    @classmethod
    def _initialise_session(self):
        self.project_folder = subprocess.check_output(
            ['git', 'rev-parse', '--show-toplevel'],
            stderr=subprocess.STDOUT).decode('utf-8').strip('\n')

        self.project = os.path.basename(self.project_folder)

        self.matador_project_folder = os.path.expanduser(
            '~/.matador/%s' % self.project)

        self.matador_repository_folder = os.path.join(
            self.matador_project_folder, 'repository')

        self.environments = get_environments(self.project_folder)

        os.makedirs(self.matador_project_folder, exist_ok=True)
        os.makedirs(self.matador_repository_folder, exist_ok=True)

        try:
            subprocess.run(
                ['git', '-C', self.matador_repository_folder, 'status'],
                stderr=subprocess.STDOUT,
                stdout=open(os.devnull, 'w'),
                check=True)
        except subprocess.CalledProcessError:
            initialise_repository(
                self.project_folder, self.matador_repository_folder)

    @classmethod
    def set_environment(self, environment):
        if self.environment is not None:
            return
        else:
            self._initialise_session()
            self.environment = self.environments[environment]
            credentials = get_credentials(self.project_folder)
            self.credentials = credentials[environment]

            self.matador_environment_folder = os.path.join(
                self.matador_project_folder, environment)
            self.matador_tickets_folder = os.path.join(
                self.matador_environment_folder, 'tickets')

            os.makedirs(self.matador_environment_folder, exist_ok=True)
            os.makedirs(self.matador_tickets_folder, exist_ok=True)

    @classmethod
    def update_repository(self):
        repo_folder = self.matador_repository_folder

        try:
            subprocess.run(
                ['git', '-C', repo_folder, 'status'],
                stderr=subprocess.STDOUT,
                stdout=open(os.devnull, 'w'),
                check=True)
        except subprocess.CalledProcessError:
            proj_folder = self.project_folder
            initialise_repository(proj_folder, repo_folder)

        subprocess.run(
            ['git', '-C', repo_folder, 'fetch', '-a'],
            stderr=subprocess.STDOUT,
            stdout=open(os.devnull, 'w'))



#!/usr/bin/env python
from .deployment import DeploymentCommand
import platform


class Hello(DeploymentCommand):

    def _execute(self):
        self._logger.info('Hello')
        self._logger.info(self.args[0])
        self._logger.info(platform.python_version())



import logging
import subprocess
import re


def substitute_keywords(text, repo_folder, commit):
    substitutions = {
        'version': commit,
        'date': subprocess.check_output(
            ['git', '-C', repo_folder, 'show', '-s', '--format=%ci', commit],
            stderr=subprocess.STDOUT).decode().strip('\n'),
    }
    new_text = ''
    for line in text.splitlines(keepends=True):
        for key, value in substitutions.items():
            rexp = '%s:.*' % key
            line = re.sub(rexp, '%s: %s' % (key, value), line)
        new_text += line
    return new_text


class DeploymentCommand(object):

    def __init__(self, *args):
        self._logger = logging.getLogger(__name__)
        self.args = args
        self._execute()

    def _execute(self):
        raise NotImplementedError



from .run_sql_script import RunSqlScript
from .deploy_ticket import DeployTicket, RemoveTicket

commands = {
    'run-sql-script': RunSqlScript,
    'deploy-ticket': DeployTicket,
    'remove-ticket': RemoveTicket
}
